# -*- coding: utf-8 -*-
from django.utils.translation import ugettext as _
from django.core.validators import validate_email
from django.core.mail import EmailMultiAlternatives
from django.core.exceptions import ValidationError
from django.template.loader import get_template
from django.template import Context
from django.http import HttpResponseRedirect

from jmb.core.middleware.force_response import ForceResponse

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from rest_framework import serializers
from rest_framework.renderers import JSONRenderer

from acomea.engagement.models import Package, FundExtra, ApplicazioneExtra, Category
from acomea.engagement.cms.models import DataSettings, EngagementNavSettings, EngagementTileSettings, \
    EngagementTitleSettings, EngagementHomepageSettings
from acomea.engagement.cms.forms import SendMail, EngagementHomepageForm


class FundExtraSerializer(serializers.ModelSerializer):
    class Meta:
        model = FundExtra
        fields = ('fund_short_name', 'fund_description', 'fund_percent_number', 'fund_color')


class ApplicazioneExtraSerializer(serializers.ModelSerializer):
    class Meta:
        model = ApplicazioneExtra
        fields = ('applicazione_short_name', 'applicazione_description')


class PackageSerializer(serializers.ModelSerializer):
    fund_extras = FundExtraSerializer(many=True, read_only=True)
    applicazione_extras = ApplicazioneExtraSerializer(many=True, read_only=True)

    class Meta:
        model = Package
        fields = (
            'id', 'period', 'timing', 'description',
            'historical_series', 'fund_extras', 'applicazione_extras'
        )


class CategorySerializer(serializers.ModelSerializer):
    packages = PackageSerializer(many=True, read_only=True)
    tile_url = serializers.Field()
    cover_url = serializers.Field()
    cover_box_url = serializers.Field()
    icon_url = serializers.Field()

    class Meta:
        model = Category
        fields = (
            'packages', 'name', 'slug', 'subtitle', 'home_description',
            'description', 'tile_url', 'cover_url', 'cover_box_url', 'icon_url'
        )


def get_package_data():
    qs = Category.objects.filter(status=1)
    serializer = CategorySerializer(qs, many=True)
    return JSONRenderer().render(serializer.data)


class EngagementTitlePlugin(CMSPluginBase):
    model = EngagementTitleSettings
    name = _("Title")
    module = _("engagement")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        context.update({
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class EngagementTilePlugin(CMSPluginBase):
    model = EngagementTileSettings
    name = _("Tile")
    module = _("engagement")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        context.update({
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class EngagementDataPlugin(CMSPluginBase):
    model = DataSettings
    name = _("Data")
    module = _("engagement")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        if user.is_authenticated():
            form_class = SendMail(initial={'email': user.email})
        else:
            form_class = SendMail
        if request.method == 'POST' and 'acomea_engagement_mail' in request.POST:
            form_class = SendMail(request.POST)
            if form_class.is_valid():
                email = request.POST.get('email')
                link_url = "{0}#{1}".format(
                    request.build_absolute_uri(),
                    request.POST.get('package')
                )
                try:
                    validate_email(email)
                    plaintext = get_template("engagement/cms_plugins/email/memo.html")
                    htmly = get_template("engagement/cms_plugins/email/memo.html")
                    d = Context({
                        'username': request.user,
                        'link': link_url
                    })
                    subject, from_email, to = \
                        'La soluzione di investimento per te', \
                        'memo@acomea.it', email
                    text_content = plaintext.render(d)
                    html_content = htmly.render(d)
                    msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
                    msg.attach_alternative(html_content, "text/html")
                    msg.send()
                    context.update({'email_message': u'Abbiamo inviato un memo alla email {0}'.format(email)})

                except ValidationError as e:
                    context.update({'err': u"L'email inserita non è valida"})
                raise ForceResponse(HttpResponseRedirect(link_url))

        self.render_template = instance.template_name
        context.update({
            'form': form_class,
            'objects': get_package_data(),
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class EngagementNavPlugin(CMSPluginBase):
    model = EngagementNavSettings
    name = _("Nav")
    module = _("engagement")
    admin_preview = False

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        context.update({
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


class EngagementHomepagePlugin(CMSPluginBase):
    model = EngagementHomepageSettings
    name = _("Homepage")
    module = _("engagement")
    admin_preview = False
    form = EngagementHomepageForm

    def render(self, context, instance, placeholder):
        self.render_template = instance.template_name
        context.update({
            'instance': instance,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(EngagementTitlePlugin)
plugin_pool.register_plugin(EngagementTilePlugin)
plugin_pool.register_plugin(EngagementDataPlugin)
plugin_pool.register_plugin(EngagementNavPlugin)
plugin_pool.register_plugin(EngagementHomepagePlugin)
