# coding: utf-8

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from easy_thumbnails.exceptions import InvalidImageFormatError

from jmb.core.db.models import UserModel, DateModel, StatusModel, OrderedModel
from imagekit.models import ProcessedImageField
from pilkit.processors import ResizeToFit
from easy_thumbnails.files import get_thumbnailer
from image_cropping import ImageRatioField
from acomea.fondi.models import Fund
from acomea.app.models import Applicazione

from jmb.core.db.models import HTMLFieldTinyMCE


class Package(UserModel, DateModel, StatusModel, OrderedModel):
    category = models.ForeignKey(
        "Category",
        related_name='packages',
        verbose_name=_("category"),
        help_text=_("Select a package category")
    )
    PERIODS = (
        (0, u'breve periodo'),
        (1, u'medio periodo'),
        (2, u'lungo periodo'),
    )
    period = models.IntegerField(
        _("period"),
        choices=PERIODS,
        help_text=_("package period")
    )
    timing = models.CharField(
        _("timing"),
        max_length=255,
        blank=True, null=True,
        help_text=_("package timing")
    )
    description = models.TextField(
        _("description'"),
        blank=True, null=True,
        help_text=_("description")
    )
    historical_series = models.TextField(
        _("historical series"),
        blank=True, null=True,
        help_text=_("paste embedded code from hightcharts")
    )
    funds = models.ManyToManyField(
        Fund,
        through='FundExtra',
        blank=True,
        related_name='package_funds',
        verbose_name=_("Funds"),
        help_text=_("Select one or more funds for this package")
    )
    applicazione = models.ManyToManyField(
        Applicazione,
        through='ApplicazioneExtra',
        blank=True,
        related_name='package_optional',
        verbose_name=_("Optional"),
        help_text=_("Select one or more funds for this package")
    )

    def __unicode__(self):
        return u'{} - {}'.format(self.category.name, self.period)

    class Meta(object):
        verbose_name = _('Package')
        verbose_name_plural = _('Packages')
        ordering = ("ordering", "category", "period",)
        unique_together = (
            ('category', 'period'),
        )


class FundExtra(UserModel, DateModel, OrderedModel):
    package = models.ForeignKey(
        Package,
        verbose_name=_('Package'),
        related_name='fund_extras'
    )
    fund = models.ForeignKey(
        Fund,
        verbose_name=_('Fund'),
        related_name='fund_extras'
    )
    fund_short_name = models.CharField(
        _("name"),
        max_length=255,
        blank=True, null=True,
        help_text=_("fund short name")
    )
    fund_description = models.TextField(
        _("fund Description'"),
        blank=True, null=True,
        help_text=_("fund description")
    )
    fund_percent_number = models.DecimalField(
        _("fund percent number"),
        max_digits=4,
        decimal_places=1,
        blank=True, null=True,
        help_text=_("fund percent number")
    )
    fund_color = models.CharField(
        blank=True, null=True,
        verbose_name=_('fund color'),
        max_length=7,
        default='#E65112',
    )

    class Meta:
        verbose_name = _('Fund extra')
        verbose_name_plural = _('Fund extras')
        ordering = ('ordering', 'fund_short_name')


class ApplicazioneExtra(UserModel, DateModel, OrderedModel):
    package = models.ForeignKey(
        Package,
        verbose_name=_('Package'),
        related_name='applicazione_extras'
    )
    applicazione = models.ForeignKey(
        Applicazione,
        verbose_name=_('Applicazione'),
        related_name='applicazione_extras'
    )
    applicazione_short_name = models.CharField(
        _("name"),
        max_length=255,
        blank=True, null=True,
        help_text=_("fund short name")
    )
    applicazione_description = models.TextField(
        _("Applicazione Description'"),
        blank=True, null=True,
        help_text=_("fund description")
    )

    class Meta:
        verbose_name = _('Applicazioni extra')
        verbose_name_plural = _('Applicazioni extras')
        ordering = ('ordering', 'applicazione_short_name')


class Category(UserModel, DateModel, StatusModel, OrderedModel):
    name = models.CharField(
        _("name"),
        max_length=255,
        blank=True, null=True,
        help_text=_("category name")
    )
    slug = models.SlugField(
        verbose_name='slug',
        max_length=255,
        blank=True, null=True
    )
    subtitle = models.CharField(
        _("subtitle"),
        max_length=255,
        blank=True, null=True,
        help_text=_("category subtitle")
    )
    home_description = HTMLFieldTinyMCE(
        _("Home description'"),
        blank=True,
        help_text=_("home description")
    )
    description = models.TextField(
        _("Description'"),
        blank=True, null=True,
        help_text=_("description")
    )
    tile = ProcessedImageField(
        processors=[ResizeToFit(width=520)],
        upload_to=settings.ENGAGEMENT_TILE_UPLOAD_TO,
        format='JPEG',
        options={'quality': 100},
        help_text=_("category tile"),
        blank=True, null=True,
        verbose_name=_("tile")
    )
    tile_cropping = ImageRatioField(
        'tile',
        '270x197',
        size_warning=True
    )
    cover = ProcessedImageField(
        processors=[ResizeToFit(width=900)],
        upload_to=settings.ENGAGEMENT_COVER_UPLOAD_TO,
        format='JPEG',
        options={'quality': 100},
        help_text=_("category cover"),
        blank=True, null=True,
        verbose_name=_("cover")
    )
    cover_cropping = ImageRatioField(
        'cover',
        '850x197',
        size_warning=True
    )
    cover_cropping_box = ImageRatioField(
        'cover',
        '200x200',
        size_warning=True
    )
    icon = ProcessedImageField(
        processors=[ResizeToFit(width=60)],
        upload_to='engagement/icons',
        format='JPEG',
        options={'quality': 100},
        help_text=_("category icon must be 30x34 and jpg format"),
        blank=True, null=True,
        verbose_name=_("icon")
    )
    icon_cropping = ImageRatioField(
        'icon',
        '30x34',
        size_warning=True
    )

    @property
    def tile_url(self):
        try:
            return get_thumbnailer(self.tile).get_thumbnail({
                'size': (270, 197),
                'box': self.tile_cropping,
                'crop': True,
                'detail': True,
            }).url
        except (InvalidImageFormatError, AttributeError):
            return ""

    @property
    def cover_url(self):
        try:
            return get_thumbnailer(self.cover).get_thumbnail({
                'size': (900, 197),
                'box': self.cover_cropping,
                'crop': True,
                'detail': True,
            }).url
        except (InvalidImageFormatError, AttributeError):
            return ""

    @property
    def cover_box_url(self):
        try:
            return get_thumbnailer(self.cover).get_thumbnail({
                'size': (200, 200),
                'box': self.cover_cropping_box,
                'crop': True,
                'detail': True,
            }).url
        except (InvalidImageFormatError, AttributeError):
            return ""

    @property
    def icon_url(self):
        try:
            return get_thumbnailer(self.icon).get_thumbnail({
                'size': (30, 34),
                'box': self.icon_cropping,
                'crop': True,
                'detail': True,
            }).url
        except (InvalidImageFormatError, AttributeError):
            return ""

    def __unicode__(self):
        return self.name

    class Meta(object):
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')
        ordering = ("ordering",)


if settings.CMS_ENABLED:
    import acomea.engagement.cms.app  # noqa
    import acomea.engagement.cms.models  # noqa
